package main

import (
	"fmt"
	"image"
	"math"
	"os"
)

const (
	Border byte = 255
	Empty  byte = 0
)

type PopulationMap struct {
	area []byte
}

type Rule struct {
	evolve  float64
	degrade float64
}

type Rules struct {
	steps int
	rules []Rule
	koefs []float64
}

func (m *PopulationMap) Copy() PopulationMap {
	p := PopulationMap{make([]byte, len(m.area))}
	copy(p.area, m.area)
	return p
}

func ReadRules(path string) Rules {
	rules := Rules{}
	file, _ := os.Open(path)

	var r int
	fmt.Fscanf(file, "%d\n", &r)
	rules.steps = r

	for range Values {
		var c float64
		fmt.Fscanf(file, "%f ", &c)
		rules.koefs = append(rules.koefs, c)
	}

	for range Values {
		var evolve, degrade float64
		fmt.Fscanf(file, "%f %f\n", &degrade, &evolve)
		rules.rules = append(rules.rules, Rule{evolve, degrade})
	}

	return rules
}

func loadOneAllMap(path string) PopulationMap {
	var oldMap PopulationMap
	imgfile, _ := os.Open(path)
	defer imgfile.Close()

	imgCfg, _, _ := image.DecodeConfig(imgfile)
	screenWidth = imgCfg.Width
	screenHeight = imgCfg.Height

	if len(oldMap.area) == 0 {
		oldMap.area = make([]byte, screenWidth*screenHeight)
	}

	imgfile.Seek(0, 0)
	img, _, _ := image.Decode(imgfile)

	for y := 0; y < screenHeight; y++ {
		for x := 0; x < screenWidth; x++ {
			r, g, b, _ := img.At(x, y).RGBA()

			r >>= 8
			g >>= 8
			b >>= 8

			distance0 := math.Sqrt(math.Pow(float64(r)-0, 2) + math.Pow(float64(g)-0, 2) + math.Pow(float64(b)-0, 2))
			distance1 := math.Sqrt(math.Pow(float64(r)-26, 2) + math.Pow(float64(g)-26, 2) + math.Pow(float64(b)-26, 2))
			distance2 := math.Sqrt(math.Pow(float64(r)-51, 2) + math.Pow(float64(g)-51, 2) + math.Pow(float64(b)-51, 2))
			distance3 := math.Sqrt(math.Pow(float64(r)-77, 2) + math.Pow(float64(g)-77, 2) + math.Pow(float64(b)-77, 2))
			distance4 := math.Sqrt(math.Pow(float64(r)-102, 2) + math.Pow(float64(g)-102, 2) + math.Pow(float64(b)-102, 2))
			distance5 := math.Sqrt(math.Pow(float64(r)-128, 2) + math.Pow(float64(g)-128, 2) + math.Pow(float64(b)-128, 2))
			distance6 := math.Sqrt(math.Pow(float64(r)-153, 2) + math.Pow(float64(g)-153, 2) + math.Pow(float64(b)-153, 2))
			distance7 := math.Sqrt(math.Pow(float64(r)-179, 2) + math.Pow(float64(g)-179, 2) + math.Pow(float64(b)-179, 2))
			distance8 := math.Sqrt(math.Pow(float64(r)-204, 2) + math.Pow(float64(g)-204, 2) + math.Pow(float64(b)-204, 2))
			distance9 := math.Sqrt(math.Pow(float64(r)-230, 2) + math.Pow(float64(g)-230, 2) + math.Pow(float64(b)-230, 2))
			distance10 := math.Sqrt(math.Pow(float64(r)-255, 2) + math.Pow(float64(g)-255, 2) + math.Pow(float64(b)-255, 2))
			distanceb := math.Sqrt(math.Pow(float64(r)-0, 2) + math.Pow(float64(g)-0, 2) + math.Pow(float64(b)-255, 2))

			min := getMin(distance0, distance1, distance2, distance3, distance4, distance5, distance6, distance7, distance8, distance9, distance10, distanceb)

			switch min {
			case distance0:
				oldMap.area[x+y*screenWidth] = byte(Values[0])
			case distance1:
				oldMap.area[x+y*screenWidth] = byte(Values[0])
			case distance2:
				oldMap.area[x+y*screenWidth] = byte(Values[1])
			case distance3:
				oldMap.area[x+y*screenWidth] = byte(Values[1])
			case distance4:
				oldMap.area[x+y*screenWidth] = byte(Values[2])
			case distance5:
				oldMap.area[x+y*screenWidth] = byte(Values[2])
			case distance6:
				oldMap.area[x+y*screenWidth] = byte(Values[3])
			case distance7:
				oldMap.area[x+y*screenWidth] = byte(Values[3])
			case distance8:
				oldMap.area[x+y*screenWidth] = byte(Values[4])
			case distance9:
				oldMap.area[x+y*screenWidth] = byte(Values[4])
			case distance10:
				oldMap.area[x+y*screenWidth] = byte(Values[5])
			case distanceb:
				oldMap.area[x+y*screenWidth] = Border
			}
		}
	}

	return oldMap
}

func getMin(values ...float64) float64 {
	min := math.MaxFloat64
	for _, v := range values {
		min = math.Min(min, v)
	}

	return min
}
